@@ -1,7 +1,12 @@
 #!/usr/bin/env perl
 #
-# Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
+# Copyright (c) 2009-2016 Cisco Systems, Inc.  All rights reserved.
 # Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
+# Copyright (c) 2013      Mellanox Technologies, Inc.
+#                         All rights reserved.
+# Copyright (c) 2015      IBM Corporation.  All rights reserved.
+# Copyright (c) 2016      Research Organization for Information Science
+#                         and Technology (RIST). All rights reserved.
 #
 # $COPYRIGHT$
 # 
@@ -41,6 +46,7 @@ my @subdirs;
 # Command line parameters
 my $no_ompi_arg = 0;
 my $no_orte_arg = 0;
+my $no_oshmem_arg = 0;
 my $quiet_arg = 0;
 my $debug_arg = 0;
 my $help_arg = 0;
@@ -53,7 +59,7 @@ my $include_list;
 my $exclude_list;
 
 # Minimum versions
-my $ompi_automake_version = "1.11.1";
+my $ompi_automake_version = "1.12.2";
 my $ompi_autoconf_version = "2.65";
 my $ompi_libtool_version = "2.2.6b";
 
@@ -177,7 +183,6 @@ sub process_subdir {
         print "--- Found configure.in|ac; running autoreconf...\n";
         safe_system("autoreconf -ivf");
         print "--- Patching autotools output... :-(\n";
-        patch_autotools_output($start);
     } else {
         my_die "Found subdir, but no autogen.sh or configure.in|ac to do anything";
     }
@@ -186,6 +191,9 @@ sub process_subdir {
     my_die "Did not generate a \"configure\" executable in $dir.\n"
         if (! -x "configure");
 
+    # Fix known issues in Autotools output
+    patch_autotools_output($start);
+
     # Chdir back to where we came from
     chdir($start);
 }
@@ -961,12 +969,40 @@ sub patch_autotools_output {
         $c =~ s/$search_string/$replace_string/;
     }
 
+    # Oracle has apparently begun (as of 12.5-beta) removing the "Sun" branding.
+    # So this patch (cumulative over the previous one) is required.
+    verbose "$indent_str"."Patching configure for Oracle Studio Fortran version strings\n";
+    $c =~ s/\*Sun\*Fortran\*\)/*Sun*Fortran* | *Studio*Fortran*)/g;
+    $c =~ s/\*Sun\\ F\*\)(.*\n\s+tmp_sharedflag=)/*Sun\\ F* | *Studio*Fortran*)$1/g;
+
     # See http://git.savannah.gnu.org/cgit/libtool.git/commit/?id=v2.2.6-201-g519bf91 for details
     # Note that this issue was fixed in LT 2.2.8, however most distros are still using 2.2.6b
 
     verbose "$indent_str"."Patching configure for IBM xlf libtool bug\n";
     $c =~ s/(\$LD -shared \$libobjs \$deplibs \$)compiler_flags( -soname \$soname)/$1linker_flags$2/g;
 
+    #Check if we are using a recent enough libtool that supports PowerPC little endian
+    if(index($c, 'powerpc64le-*linux*)') == -1) {
+        verbose "$indent_str"."Patching configure for PowerPC little endian support\n";
+        my $replace_string = "x86_64-*kfreebsd*-gnu|x86_64-*linux*|powerpc*-*linux*|";
+        $c =~ s/x86_64-\*kfreebsd\*-gnu\|x86_64-\*linux\*\|ppc\*-\*linux\*\|powerpc\*-\*linux\*\|/$replace_string/g;
+        $replace_string =
+        "powerpc64le-*linux*)\n\t    LD=\"\${LD-ld} -m elf32lppclinux\"\n\t    ;;\n\t  powerpc64-*linux*)";
+        $c =~ s/ppc64-\*linux\*\|powerpc64-\*linux\*\)/$replace_string/g;
+        $replace_string =
+        "powerpcle-*linux*)\n\t    LD=\"\${LD-ld} -m elf64lppc\"\n\t    ;;\n\t  powerpc-*linux*)";
+        $c =~ s/ppc\*-\*linux\*\|powerpc\*-\*linux\*\)/$replace_string/g;
+    }
+
+    # Fix consequence of broken libtool.m4
+    # see http://lists.gnu.org/archive/html/bug-libtool/2015-07/msg00002.html and
+    # https://github.com/open-mpi/ompi/issues/751
+    verbose "$indent_str"."Patching configure for libtool.m4 bug\n";
+    # patch for libtool < 2.4.3
+    $c =~ s/# Some compilers place space between "-\{L,R\}" and the path.\n       # Remove the space.\n       if test \$p = \"-L\" \|\|/# Some compilers place space between "-\{L,-l,R\}" and the path.\n       # Remove the spaces.\n       if test \$p = \"-L\" \|\|\n          test \$p = \"-l\" \|\|/g;
+    # patch for libtool >= 2.4.3
+    $c =~ s/# Some compilers place space between "-\{L,R\}" and the path.\n       # Remove the space.\n       if test x-L = \"\$p\" \|\|\n          test x-R = \"\$p\"\; then/# Some compilers place space between "-\{L,-l,R\}" and the path.\n       # Remove the spaces.\n       if test x-L = \"x\$p\" \|\|\n          test x-l = \"x\$p\" \|\|\n          test x-R = \"x\$p\"\; then/g;
+
     open(OUT, ">configure.patched") || my_die "Can't open configure.patched";
     print OUT $c;
     close(OUT);
@@ -985,6 +1021,7 @@ sub patch_autotools_output {
 
 my $ok = Getopt::Long::GetOptions("no-ompi" => \$no_ompi_arg,
                                   "no-orte" => \$no_orte_arg,
+                                  "no-oshmem" => \$no_oshmem_arg,
                                   "quiet|q" => \$quiet_arg,
                                   "debug|d" => \$debug_arg,
                                   "help|h" => \$help_arg,
@@ -999,6 +1036,7 @@ if (!$ok || $help_arg) {
     print "Options:
   --no-ompi | -no-ompi          Do not build the Open MPI layer
   --no-orte | -no-orte          Do not build the ORTE layer
+  --no-oshmem | -no-oshmem      Do not build the OSHMEM layer
   --quiet | -q                  Do not display normal verbose output
   --debug | -d                  Output lots of debug information
   --help | -h                   This help list
@@ -1027,6 +1065,10 @@ if (! -e "orte") {
     $no_orte_arg = 1;
     debug "No orte subdirectory found - will not build ORTE\n";
 }
+if (! -e "oshmem") {
+    $no_oshmem_arg = 1;
+    debug "No oshmem subdirectory found - will not build OSHMEM\n";
+}
 
 if ($no_ompi_arg == 1 && $no_orte_arg == 0) {
     $project_name_long = "Open MPI Run Time Environment";
@@ -1193,6 +1235,8 @@ push(@{$projects}, { name => "orte", dir => "orte", need_base => 1 })
     if (!$no_orte_arg);
 push(@{$projects}, { name => "ompi", dir => "ompi", need_base => 1 })
     if (!$no_ompi_arg);
+push(@{$projects}, { name => "oshmem", dir => "oshmem", need_base => 1 })
+    if (!$no_ompi_arg && !$no_orte_arg && !$no_oshmem_arg);
 
 $m4 .= "dnl Separate m4 define for each project\n";
 foreach my $p (@$projects) {
@@ -1242,7 +1286,6 @@ verbose "\n$step. Running autotools on top-level tree\n\n";
 verbose "==> Remove stale files\n";
 find_and_delete(qw/config.guess config.sub depcomp compile install-sh ltconfig
     ltmain.sh missing mkinstalldirs libtool/);
-system("rm -rf opal/libltdl");
 
 # Remove the old m4 file and write the new one
 verbose "==> Writing m4 file with autogen.pl results\n";
@@ -1267,24 +1310,6 @@ foreach my $project (@{$projects}) {
 }
 safe_system($cmd);
 
-#---------------------------------------------------------------------------
-
-# For FreeBSD (carried over from autogen.sh); apparently some versions
-# of automake don't so this (prior to 1.9.7...?).
-system("chmod u+w opal/libltdl/configure");
-
-#---------------------------------------------------------------------------
-
-++$step;
-verbose "\n$step. Patching autotools output on top-level tree :-(\n\n";
-
-# Patch preopen error in libltdl
-if (-f "opal/libltdl/loaders/preopen.c") {
-    verbose "=== Patching preopen error masking in libltdl\n";
-    safe_system("$patch_prog -N -p0 < config/libltdl-preopen-error.diff");
-    unlink("opal/libltdl/loaders/preopen.c.rej");
-}
-
 patch_autotools_output(".");
 
 #---------------------------------------------------------------------------
