@@ -132,7 +132,7 @@ void hcoll_rte_fns_setup(void)
                 0, 0,
                 /* NOTE: hack - need to parametrize this */
                 10,
-                50,
+                -1,
                 10,
                 /* No Mpool */
                 NULL,
@@ -398,7 +398,9 @@ static void* get_coll_handle(void)
     OMPI_REQUEST_INIT(ompi_req,false);
     ompi_req->req_complete_cb = NULL;
     ompi_req->req_status.MPI_ERROR = MPI_SUCCESS;
+    ompi_req->req_state = OMPI_REQUEST_ACTIVE;
     ompi_req->req_free = request_free;
+    ompi_req->req_type = OMPI_REQUEST_COLL;
     return (void *)ompi_req;
 }
 
