@@ -10,12 +10,13 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006      Voltaire. All rights reserved.
  * Copyright (c) 2007      Mellanox Technologies. All rights reserved.
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2013      NVIDIA Corporation.  All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -29,12 +30,13 @@
 
 #include <errno.h>
 #include <string.h>
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#endif
+#include <stdlib.h>
 
 #include "opal/align.h"
 
+#if OPAL_CUDA_GDR_SUPPORT
+#include "ompi/mca/common/cuda/common_cuda.h"
+#endif /* OPAL_CUDA_GDR_SUPPORT */
 #include "ompi/mca/rcache/rcache.h"
 #include "ompi/mca/rcache/base/base.h"
 #include "ompi/mca/rte/rte.h"
@@ -43,6 +45,9 @@
 #include "ompi/mca/mpool/base/base.h"
 #include "mpool_grdma.h"
 
+#if OPAL_CUDA_GDR_SUPPORT
+static int check_for_cuda_freed_memory(mca_mpool_base_module_t *mpool, void *addr, size_t size);
+#endif /* OPAL_CUDA_GDR_SUPPORT */
 static void mca_mpool_grdma_pool_contructor (mca_mpool_grdma_pool_t *pool)
 {
     memset ((void *)((uintptr_t)pool + sizeof (pool->super)), 0, sizeof (*pool) - sizeof (pool->super));
@@ -130,7 +135,7 @@ void* mca_mpool_grdma_alloc(mca_mpool_base_module_t *mpool, size_t size,
     if(0 == align)
         align = mca_mpool_base_page_size;
 
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
     /* CUDA cannot handle registering overlapping regions, so make
      * sure each region is page sized and page aligned. */
     align = mca_mpool_base_page_size;
@@ -229,11 +234,22 @@ int mca_mpool_grdma_register(mca_mpool_base_module_t *mpool, void *addr,
     if (!opal_list_is_empty (&mpool_grdma->pool->gc_list))
         do_unregistration_gc(mpool);
 
+#if OPAL_CUDA_GDR_SUPPORT
+    if (flags & MCA_MPOOL_FLAGS_CUDA_GPU_MEM) {
+        size_t psize;
+        mca_common_cuda_get_address_range(&base, &psize, addr);
+        bound = base + psize - 1;
+        /* Check to see if this memory is in the cache and if it has been freed. If so,
+         * this call will boot it out of the cache. */
+        check_for_cuda_freed_memory(mpool, base, psize);
+    }
+#endif /* OPAL_CUDA_GDR_SUPPORT */
+
     /* look through existing regs if not persistent registration requested.
      * Persistent registration are always registered and placed in the cache */
     if(!(bypass_cache || persist)) {
         /* check to see if memory is registered */
-        mpool->rcache->rcache_find(mpool->rcache, addr, size, reg);
+        mpool->rcache->rcache_find(mpool->rcache, base, bound - base + 1, reg);
         if (*reg && !(flags & MCA_MPOOL_FLAGS_INVALID)) {
             if (0 == (*reg)->ref_count) {
                 /* Leave pinned must be set for this to still be in the rcache. */
@@ -269,6 +285,11 @@ int mca_mpool_grdma_register(mca_mpool_base_module_t *mpool, void *addr,
     grdma_reg->base = base;
     grdma_reg->bound = bound;
     grdma_reg->flags = flags;
+#if OPAL_CUDA_GDR_SUPPORT
+    if (flags & MCA_MPOOL_FLAGS_CUDA_GPU_MEM) {
+        mca_common_cuda_get_buffer_id(grdma_reg);
+    }
+#endif /* OPAL_CUDA_GDR_SUPPORT */
 
     if (false == bypass_cache) {
         rc = mpool->rcache->rcache_insert(mpool->rcache, grdma_reg, 0);
@@ -346,7 +367,7 @@ int mca_mpool_grdma_find(struct mca_mpool_base_module_t *mpool, void *addr,
 
     OPAL_THREAD_LOCK(&mpool->rcache->lock);
 
-    rc = mpool->rcache->rcache_find(mpool->rcache, addr, size, reg);
+    rc = mpool->rcache->rcache_find(mpool->rcache, base, bound - base + 1, reg);
     if(NULL != *reg &&
             (mca_mpool_grdma_component.leave_pinned ||
              ((*reg)->flags & MCA_MPOOL_FLAGS_PERSIST) ||
@@ -439,6 +460,61 @@ int mca_mpool_grdma_release_memory(struct mca_mpool_base_module_t *mpool,
     return rc;
 }
 
+/* Make sure this registration request is not stale.  In other words, ensure
+ * that we do not have a cuMemAlloc, cuMemFree, cuMemAlloc state.  If we do
+ * kick out the regisrations and deregister.  This function needs to be called
+ * with the mpool->rcache->lock held. */
+#if OPAL_CUDA_GDR_SUPPORT
+static int check_for_cuda_freed_memory(mca_mpool_base_module_t *mpool, void *addr, size_t size)
+{
+    mca_mpool_grdma_module_t *mpool_grdma = (mca_mpool_grdma_module_t *) mpool;
+    mca_mpool_base_registration_t *regs[GRDMA_MPOOL_NREGS];
+    int reg_cnt, i, rc = OMPI_SUCCESS;
+    mca_mpool_base_registration_t *reg;
+
+    mpool->rcache->rcache_find(mpool->rcache, addr, size, &reg);
+    if (NULL == reg) {
+        return OMPI_SUCCESS;
+    }
+
+    /* If not previously freed memory, just return 0 */
+    if (!(mca_common_cuda_previously_freed_memory(reg))) {
+        return OMPI_SUCCESS;
+    }
+
+    /* mpool->rcache->rcache_dump_range(mpool->rcache, 0, (size_t)-1, "Before free"); */
+
+    /* This memory has been freed.  Find all registrations and delete */
+    do {
+        reg_cnt = mpool->rcache->rcache_find_all(mpool->rcache, reg->base, reg->bound - reg->base + 1,
+                                                 regs, GRDMA_MPOOL_NREGS);
+        for(i = 0 ; i < reg_cnt ; ++i) {
+            regs[i]->flags |= MCA_MPOOL_FLAGS_INVALID;
+            if (regs[i]->ref_count) {
+                opal_output(0, "Release FAILED: ref_count=%d, base=%p, bound=%p, size=%d",
+                            regs[i]->ref_count, regs[i]->base, regs[i]->bound,
+                            (int) (regs[i]->bound - regs[i]->base + 1));
+                /* memory is being freed, but there are registration in use that
+                 * covers the memory. This can happen even in a correct program,
+                 * but may also be an user error. We can't tell. Mark the
+                 * registration as invalid. It will not be used any more and
+                 * will be unregistered when ref_count will become zero */
+                rc = OMPI_ERROR; /* tell caller that something was wrong */
+            } else {
+                opal_list_remove_item(&mpool_grdma->pool->lru_list,(opal_list_item_t *) regs[i]);
+                /* Now deregister.  Do not use gc_list as we need to kick this out now. */
+                dereg_mem(regs[i]);
+            }
+        }
+    } while(reg_cnt == GRDMA_MPOOL_NREGS);
+
+    OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
+    /* mpool->rcache->rcache_dump_range(mpool->rcache, 0, (size_t)-1, "After free");*/
+
+    return rc;
+}
+#endif /* OPAL_CUDA_GDR_SUPPORT */
+
 void mca_mpool_grdma_finalize(struct mca_mpool_base_module_t *mpool)
 {
     mca_mpool_grdma_module_t *mpool_grdma = (mca_mpool_grdma_module_t*)mpool;
@@ -482,6 +558,9 @@ void mca_mpool_grdma_finalize(struct mca_mpool_base_module_t *mpool)
 
     /* Cleanup any vmas that we have deferred deletion on */
     mpool->rcache->rcache_clean(mpool->rcache);
+
+    /* this mpool was allocated by grdma_init in mpool_grdma_component.c */
+    free(mpool);
 }
 
 int mca_mpool_grdma_ft_event(int state) {
