@@ -5,7 +5,7 @@
  *                         reserved.
  * Copyright (c) 2014      Intel, Inc. All rights reserved.
  * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  * 
@@ -60,45 +60,45 @@ ompi_osc_sm_component_t mca_osc_sm_component = {
 
 ompi_osc_sm_module_t ompi_osc_sm_module_template = {
     {
-        ompi_osc_sm_shared_query,
+        .osc_win_shared_query = ompi_osc_sm_shared_query,
 
-        ompi_osc_sm_attach,
-        ompi_osc_sm_detach,
-        ompi_osc_sm_free,
+        .osc_win_attach = ompi_osc_sm_attach,
+        .osc_win_detach = ompi_osc_sm_detach,
+        .osc_free = ompi_osc_sm_free,
 
-        ompi_osc_sm_put,
-        ompi_osc_sm_get,
-        ompi_osc_sm_accumulate,
-        ompi_osc_sm_compare_and_swap,
-        ompi_osc_sm_fetch_and_op,
-        ompi_osc_sm_get_accumulate,
+        .osc_put = ompi_osc_sm_put,
+        .osc_get = ompi_osc_sm_get,
+        .osc_accumulate = ompi_osc_sm_accumulate,
+        .osc_compare_and_swap = ompi_osc_sm_compare_and_swap,
+        .osc_fetch_and_op = ompi_osc_sm_fetch_and_op,
+        .osc_get_accumulate = ompi_osc_sm_get_accumulate,
 
-        ompi_osc_sm_rput,
-        ompi_osc_sm_rget,
-        ompi_osc_sm_raccumulate,
-        ompi_osc_sm_rget_accumulate,
+        .osc_rput = ompi_osc_sm_rput,
+        .osc_rget = ompi_osc_sm_rget,
+        .osc_raccumulate = ompi_osc_sm_raccumulate,
+        .osc_rget_accumulate = ompi_osc_sm_rget_accumulate,
 
-        ompi_osc_sm_fence,
+        .osc_fence = ompi_osc_sm_fence,
 
-        ompi_osc_sm_start,
-        ompi_osc_sm_complete,
-        ompi_osc_sm_post,
-        ompi_osc_sm_wait,
-        ompi_osc_sm_test,
+        .osc_start = ompi_osc_sm_start,
+        .osc_complete = ompi_osc_sm_complete,
+        .osc_post = ompi_osc_sm_post,
+        .osc_wait = ompi_osc_sm_wait,
+        .osc_test = ompi_osc_sm_test,
 
-        ompi_osc_sm_lock,
-        ompi_osc_sm_unlock,
-        ompi_osc_sm_lock_all,
-        ompi_osc_sm_unlock_all,
+        .osc_lock = ompi_osc_sm_lock,
+        .osc_unlock = ompi_osc_sm_unlock,
+        .osc_lock_all = ompi_osc_sm_lock_all,
+        .osc_unlock_all = ompi_osc_sm_unlock_all,
 
-        ompi_osc_sm_sync,
-        ompi_osc_sm_flush,
-        ompi_osc_sm_flush_all,
-        ompi_osc_sm_flush_local,
-        ompi_osc_sm_flush_local_all,
+        .osc_sync = ompi_osc_sm_sync,
+        .osc_flush = ompi_osc_sm_flush,
+        .osc_flush_all = ompi_osc_sm_flush_all,
+        .osc_flush_local = ompi_osc_sm_flush_local,
+        .osc_flush_local_all = ompi_osc_sm_flush_local_all,
 
-        ompi_osc_sm_set_info,
-        ompi_osc_sm_get_info
+        .osc_set_info = ompi_osc_sm_set_info,
+        .osc_get_info = ompi_osc_sm_get_info
     }
 };
 
@@ -169,6 +169,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
                  int flavor, int *model)
 {
     ompi_osc_sm_module_t *module = NULL;
+    int comm_size = ompi_comm_size (comm);
     int ret = OMPI_ERROR;
 
     if (OMPI_SUCCESS != (ret = check_win_ok(comm, flavor))) {
@@ -180,6 +181,8 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         calloc(1, sizeof(ompi_osc_sm_module_t));
     if (NULL == module) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
+    OBJ_CONSTRUCT(&module->lock, opal_mutex_t);
+
     /* fill in the function pointer part */
     memcpy(module, &ompi_osc_sm_module_template, 
            sizeof(ompi_osc_base_module_t));
@@ -191,7 +194,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     module->flavor = flavor;
 
     /* create the segment */
-    if (1 == ompi_comm_size(comm)) {
+    if (1 == comm_size) {
         module->segment_base = NULL;
         module->sizes = malloc(sizeof(size_t));
         if (NULL == module->sizes) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
@@ -206,13 +209,15 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         if (NULL == module->global_state) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
         module->node_states = malloc(sizeof(ompi_osc_sm_node_state_t));
         if (NULL == module->node_states) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-
+        module->posts = calloc (1, sizeof(module->posts[0]) + sizeof (uint64_t));
+        if (NULL == module->posts) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
+        module->posts[0] = (uint64_t *) (module->posts + 1);
     } else {
         unsigned long total, *rbuf;
-        char *data_file;
         int i, flag;
         size_t pagesize;
-	size_t state_size;
+        size_t state_size;
+        int posts_size, post_size = (comm_size + 63) / 64;
 
         OPAL_OUTPUT_VERBOSE((1, ompi_osc_base_framework.framework_output,
                              "allocating shared memory region of size %ld\n", (long) size));
@@ -220,7 +225,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         /* get the pagesize */
         pagesize = opal_getpagesize();
 
-        rbuf = malloc(sizeof(unsigned long) * ompi_comm_size(module->comm));
+        rbuf = malloc(sizeof(unsigned long) * comm_size);
         if (NULL == rbuf) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
         module->noncontig = false;
@@ -241,25 +246,28 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         if (OMPI_SUCCESS != ret) return ret;
 
         total = 0;
-        for (i = 0 ; i < ompi_comm_size(module->comm) ; ++i) {
+        for (i = 0 ; i < comm_size ; ++i) {
             total += rbuf[i];
         }
 
-        if (asprintf(&data_file, "%s"OPAL_PATH_SEP"shared_window_%d.%s",
-                     ompi_process_info.job_session_dir,
-                     ompi_comm_get_cid(module->comm),
-                     ompi_process_info.nodename) < 0) {
-            return OMPI_ERR_OUT_OF_RESOURCE;
-        }
-
 	/* user opal/shmem directly to create a shared memory segment */
-	state_size = sizeof(ompi_osc_sm_global_state_t) + sizeof(ompi_osc_sm_node_state_t) * ompi_comm_size(module->comm);
-	if (0 == ompi_comm_rank (module->comm)) {
-	    ret = opal_shmem_segment_create (&module->seg_ds, data_file, total + pagesize + state_size);
-	    if (OPAL_SUCCESS != ret) {
-		goto error;
-	    }
-	}
+	state_size = sizeof(ompi_osc_sm_global_state_t) + sizeof(ompi_osc_sm_node_state_t) * comm_size;
+        posts_size = comm_size * post_size * sizeof (uint64_t);
+        if (0 == ompi_comm_rank (module->comm)) {
+            char *data_file;
+            if (asprintf(&data_file, "%s"OPAL_PATH_SEP"shared_window_%d.%s",
+                         ompi_process_info.proc_session_dir,
+                         ompi_comm_get_cid(module->comm),
+                         ompi_process_info.nodename) < 0) {
+                return OMPI_ERR_OUT_OF_RESOURCE;
+            }
+
+            ret = opal_shmem_segment_create (&module->seg_ds, data_file, total + pagesize + state_size + posts_size);
+            free(data_file);
+            if (OPAL_SUCCESS != ret) {
+                goto error;
+            }
+        }
 
 	ret = module->comm->c_coll.coll_bcast (&module->seg_ds, sizeof (module->seg_ds), MPI_BYTE, 0,
 					       module->comm, module->comm->c_coll.coll_bcast_module);
@@ -272,15 +280,23 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 	    goto error;
 	}
 
-        module->sizes = malloc(sizeof(size_t) * ompi_comm_size(module->comm));
+        module->sizes = malloc(sizeof(size_t) * comm_size);
         if (NULL == module->sizes) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-        module->bases = malloc(sizeof(void*) * ompi_comm_size(module->comm));
+        module->bases = malloc(sizeof(void*) * comm_size);
         if (NULL == module->bases) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
+        module->posts = calloc (comm_size, sizeof (module->posts[0]));
+        if (NULL == module->posts) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
-        module->global_state = (ompi_osc_sm_global_state_t *) (module->segment_base);
+        /* set module->posts[0] first to ensure 64-bit alignment */
+        module->posts[0] = (uint64_t *) (module->segment_base);
+        module->global_state = (ompi_osc_sm_global_state_t *) (module->posts[0] + comm_size * post_size);
         module->node_states = (ompi_osc_sm_node_state_t *) (module->global_state + 1);
 
-        for (i = 0, total = state_size ; i < ompi_comm_size(module->comm) ; ++i) {
+        for (i = 0, total = state_size + posts_size ; i < comm_size ; ++i) {
+            if (i > 0) {
+                module->posts[i] = module->posts[i - 1] + post_size;
+            }
+
             module->sizes[i] = rbuf[i];
             if (module->sizes[i]) {
                 module->bases[i] = ((char *) module->segment_base) + total;
@@ -302,7 +318,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     opal_atomic_init(&module->my_node_state->accumulate_lock, OPAL_ATOMIC_UNLOCKED);
 
     /* share everyone's displacement units. */
-    module->disp_units = malloc(sizeof(int) * ompi_comm_size(module->comm));
+    module->disp_units = malloc(sizeof(int) * comm_size);
     ret = module->comm->c_coll.coll_allgather(&disp_unit, 1, MPI_INT,
                                               module->disp_units, 1, MPI_INT,
                                               module->comm,
@@ -315,7 +331,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     /* initialize synchronization code */
     module->my_sense = 1;
 
-    module->outstanding_locks = calloc(ompi_comm_size(module->comm), sizeof(enum ompi_osc_sm_locktype_t));
+    module->outstanding_locks = calloc(comm_size, sizeof(enum ompi_osc_sm_locktype_t));
     if (NULL == module->outstanding_locks) {
         ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
         goto error;
@@ -352,7 +368,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
             }
             module->global_state->use_barrier_for_fence = 0;
             module->global_state->sense = module->my_sense;
-            module->global_state->count = ompi_comm_size(module->comm);
+            module->global_state->count = comm_size;
             pthread_mutexattr_destroy(&mattr);
         } else {
             module->global_state->use_barrier_for_fence = 1;
@@ -373,8 +389,8 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     return OMPI_SUCCESS;
 
  error:
-    if (NULL != module->comm) ompi_comm_free(&module->comm);
-    if (NULL != module) free(module);
+    win->w_osc_module = &module->super;
+    ompi_osc_sm_free (win);
 
     return ret;
 }
@@ -465,8 +481,13 @@ ompi_osc_sm_free(struct ompi_win_t *win)
         free(module->sizes);
     }
 
+    free (module->posts);
+
     /* cleanup */
     ompi_comm_free(&module->comm);
+
+    OBJ_DESTRUCT(&module->lock);
+
     free(module);
 
     return OMPI_SUCCESS;
