@@ -15,7 +15,7 @@
  * Copyright (c) 2009      Institut National de Recherche en Informatique
  *                         et Automatique. All rights reserved.
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved. 
- * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2015 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -124,6 +124,7 @@ static struct {
     bool abort;
     bool mapreduce;
     bool tree_spawn;
+    char *hnp_topo_sig;
 } orted_globals;
 
 /*
@@ -215,6 +216,10 @@ opal_cmd_line_init_t orte_cmd_line_opts[] = {
     { "orte_hetero_nodes", '\0', NULL, "hetero-nodes", 0,
       NULL, OPAL_CMD_LINE_TYPE_BOOL,
       "Nodes in cluster may differ in topology, so send the topology back from each node [Default = false]" },
+    
+    { NULL, '\0', NULL, "hnp-topo-sig", 1,
+      &orted_globals.hnp_topo_sig, OPAL_CMD_LINE_TYPE_STRING,
+      "Topology signature of HNP" },
 #endif
 
     { NULL, '\0', "mapreduce", "mapreduce", 0,
@@ -389,7 +394,7 @@ int orte_daemon(int argc, char *argv[])
             res = hwloc_bitmap_alloc();
             for (i=0; NULL != cores[i]; i++) {
                 core = strtoul(cores[i], NULL, 10);
-                if (NULL == (pu = opal_hwloc_base_get_pu(opal_hwloc_topology, core))) {
+                if (NULL == (pu = opal_hwloc_base_get_pu(opal_hwloc_topology, core, OPAL_HWLOC_LOGICAL))) {
                     /* turn off the show help forwarding as we won't
                      * be able to cycle the event library to send
                      */
@@ -778,12 +783,27 @@ int orte_daemon(int argc, char *argv[])
 #if OPAL_HAVE_HWLOC
         {
             char *coprocessors;
-            /* add the local topology */
-            if (NULL != opal_hwloc_topology &&
-                (1 == ORTE_PROC_MY_NAME->vpid || orte_hetero_nodes)) {
+            uint8_t tflag;
+                        
+            /* add the local topology, if different from the HNP's or user directed us to,
+             * but always if we are the first daemon to ensure we get a compute node */
+            if (1 == ORTE_PROC_MY_NAME->vpid || orte_hetero_nodes ||
+                0 != strcmp(orte_topo_signature, orted_globals.hnp_topo_sig)) {
+                tflag = 1;
+                if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &tflag, 1, OPAL_UINT8))) {
+                    ORTE_ERROR_LOG(ret);
+                }
+                if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &orte_topo_signature, 1, OPAL_STRING))) {
+                    ORTE_ERROR_LOG(ret);
+                }
                 if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &opal_hwloc_topology, 1, OPAL_HWLOC_TOPO))) {
                     ORTE_ERROR_LOG(ret);
                 }
+            } else {
+                tflag = 0;
+                if (ORTE_SUCCESS != (ret = opal_dss.pack(buffer, &tflag, 1, OPAL_UINT8))) {
+                    ORTE_ERROR_LOG(ret);
+                }
             }
             /* detect and add any coprocessors */
             coprocessors = opal_hwloc_base_find_coprocessors(opal_hwloc_topology);
