@@ -17,6 +17,7 @@
 #include "oshmem/mca/atomic/atomic.h"
 #include "oshmem/mca/atomic/base/base.h"
 #include "oshmem/mca/memheap/memheap.h"
+#include "oshmem/mca/memheap/base/base.h"
 #include "oshmem/runtime/runtime.h"
 
 #include "atomic_mxm.h"
@@ -40,18 +41,6 @@ int mca_atomic_mxm_cswap(void *target,
     ptl_id = -1;
     mxm_err = MXM_OK;
 
-    if (!prev || !target || !value) {
-        ATOMIC_ERROR("[#%d] Whether target, value or prev are not defined",
-                     my_pe);
-        oshmem_shmem_abort(-1);
-        return OSHMEM_ERR_BAD_PARAM;
-    }
-    if ((pe < 0) || (pe >= oshmem_num_procs())) {
-        ATOMIC_ERROR("[#%d] PE=%d not valid", my_pe, pe);
-        oshmem_shmem_abort(-1);
-        return OSHMEM_ERR_BAD_PARAM;
-    }
-
     switch (nlong) {
     case 1:
         nlong_order = 0;
@@ -75,10 +64,7 @@ int mca_atomic_mxm_cswap(void *target,
     if (MXM_PTL_SHM == ptl_id) {
         ptl_id = MXM_PTL_RDMA;
     }
-    r_mkey = mca_memheap.memheap_get_cached_mkey(pe,
-                                             target,
-                                             ptl_id,
-                                             &remote_addr);
+    r_mkey = mca_memheap_base_get_cached_mkey(pe, target, ptl_id, &remote_addr);
     if (!r_mkey) {
         ATOMIC_ERROR("[#%d] %p is not address of symmetric variable",
                      my_pe, target);
