@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2013      Mellanox Technologies, Inc.
+ * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
  * 
  * $HEADER$
@@ -23,15 +23,35 @@
 #include "oshmem/runtime/runtime.h"
 #include "oshmem/shmem/shmem_api_logger.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/include/pshmem.h"
+#pragma weak shmem_init = pshmem_init
+#pragma weak start_pes = pstart_pes
+#include "oshmem/shmem/c/profile/defines.h"
+#endif
+
+extern int oshmem_shmem_globalexit_status;
+
+static inline void _shmem_init(void);
+
+void shmem_init(void)
+{
+    /* spec says that npes are ignored for now */
+    _shmem_init();
+}
+
 void start_pes(int npes)
 {
     /* spec says that npes are ignored for now */
-    shmem_init();
+    _shmem_init();
 }
 
-#if !defined(OSHMEM_PROFILING) || (OSHMEM_PROFILING == 0)
+static void shmem_onexit(int exitcode, void *arg)
+{
+    oshmem_shmem_globalexit_status = exitcode;
+}
 
-void shmem_init(void)
+static inline void _shmem_init(void)
 {
     int err = OSHMEM_SUCCESS;
     int provided;
@@ -52,7 +72,8 @@ void shmem_init(void)
     }
 
     OPAL_CR_INIT_LIBRARY();
+#if HAVE_ON_EXIT
+    on_exit(shmem_onexit, NULL);
+#endif
 }
 
-#endif /* OSHMEM_PROFILING */
-
